"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RedisCache = void 0;
const redis_1 = require("redis");
const exit_1 = require("../../exit");
const instance_1 = require("../../instance");
const cache_1 = require("../cache");
class RedisCache extends cache_1.Cache {
    client;
    constructor() {
        super();
        this.client = (0, redis_1.createClient)({ url: instance_1.Instance.get().settings.redisURI });
        this.client.on('error', async (error) => {
            (0, exit_1.exit)(`Redis failed with error: ${error}`);
        });
    }
    async start() {
        await this.client.connect();
    }
    async close() {
        this.client.quit();
    }
    async delete(key) {
        await this.client.del(key);
    }
    async get(key) {
        return await this.client.get(key);
    }
    async set(key, data, ttlInSecs) {
        if (ttlInSecs > 0)
            await this.client.setEx(key, ttlInSecs, data);
        else
            this.client.set(key, data);
    }
    async getOrSet(key, fn, ttlInSecs) {
        const cached = await this.get(key);
        if (cached)
            return JSON.parse(cached);
        const result = await fn();
        await this.set(key, JSON.stringify(result), ttlInSecs);
    }
}
exports.RedisCache = RedisCache;
