import { BaseEntity } from '../structure';
import { DeepPartial } from '../types';
import { DebeziumSetup } from './debezium';
import { QueryParams, QueryResults } from './query';
export declare abstract class Db {
    #private;
    abstract change<Model, Entity extends BaseEntity<any, any>>(collection: any, callbacks: DbChangeCallbacks<Model, Entity>, mapper: (model: Model | null) => Entity | null): DbChange<Model, Entity>;
    abstract query<Model>(collection: any, params: QueryParams): Promise<QueryResults<Model>>;
    protected _addToDbChanges(dbChange: DbChange<any, any>): this;
    startAllDbChanges(): Promise<void>;
    abstract start(): Promise<void>;
    abstract close(): Promise<void>;
}
export declare abstract class DbChange<Model, Entity extends BaseEntity<any, any>> {
    #private;
    constructor(callbacks: DbChangeCallbacks<Model, Entity>, mapper: (model: Model | null) => Entity | null);
    abstract start(): Promise<void>;
    get callbacks(): Readonly<DbChangeCallbacks<Model, Entity>>;
    get mapper(): (model: Model | null) => Entity | null;
    protected _setup(key: string, data: DebeziumSetup): Promise<boolean>;
}
export type DbChangeCallbacks<Model, Entity> = {
    created?: (data: {
        before: null;
        after: Entity;
    }) => Promise<void>;
    updated?: (data: {
        before: Entity;
        after: Entity;
        changes: DeepPartial<Model>;
    }) => Promise<void>;
    deleted?: (data: {
        before: Entity;
        after: null;
    }) => Promise<void>;
};
