"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DbChange = exports.Db = void 0;
const axios_1 = __importDefault(require("axios"));
const exit_1 = require("../exit");
const instance_1 = require("../instance");
const debezium_1 = require("./debezium");
class Db {
    #dbChanges = [];
    _addToDbChanges(dbChange) {
        this.#dbChanges.push(dbChange);
        return this;
    }
    async startAllDbChanges() {
        await Promise.all(this.#dbChanges.map((change) => change.start()));
    }
}
exports.Db = Db;
class DbChange {
    #callbacks = {};
    #mapper;
    constructor(callbacks, mapper) {
        this.#callbacks = callbacks;
        this.#mapper = mapper;
    }
    get callbacks() {
        return Object.freeze(this.#callbacks);
    }
    get mapper() {
        return this.#mapper;
    }
    async _setup(key, data) {
        data = { ...debezium_1.DefaultDebeziumSetup, ...data };
        const topics = await axios_1.default.put(`/connectors/${key}/config`, data, { baseURL: instance_1.Instance.get().settings.debeziumUrl })
            .then(async () => {
            const res = await axios_1.default.get(`/connectors/${key}/topics`, { baseURL: instance_1.Instance.get().settings.debeziumUrl });
            return res.data[key]?.topics ?? [];
        })
            .catch((err) => {
            (0, exit_1.exit)(`Failed to setup debezium for ${key}: ${err.message}`);
            return [];
        });
        return topics[0] === key;
    }
}
exports.DbChange = DbChange;
