"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongoDb = void 0;
const mongoose_1 = __importDefault(require("mongoose"));
const mongoose_lean_defaults_1 = __importDefault(require("mongoose-lean-defaults"));
const mongoose_lean_getters_1 = __importDefault(require("mongoose-lean-getters"));
const mongoose_lean_virtuals_1 = __importDefault(require("mongoose-lean-virtuals"));
const instance_1 = require("../../instance");
const _instance_1 = require("../_instance");
const changes_1 = require("./changes");
const query_1 = require("./query");
class MongoDb extends _instance_1.Db {
    #started = false;
    get Schema() {
        return mongoose_1.default.Schema;
    }
    get Id() {
        return new mongoose_1.default.Types.ObjectId();
    }
    get #connections() {
        return mongoose_1.default.connection.otherDbs ?? [];
    }
    use(dbName = 'default') {
        const conn = dbName === 'default' ? mongoose_1.default.connection : mongoose_1.default.connection.useDb(dbName, { useCache: true });
        conn.plugin(mongoose_lean_defaults_1.default).plugin(mongoose_lean_virtuals_1.default).plugin(mongoose_lean_getters_1.default);
        return conn;
    }
    change(model, callbacks, mapper) {
        const change = new changes_1.MongoDbChange(model, callbacks, mapper);
        this._addToDbChanges(change);
        return change;
    }
    async query(model, params) {
        return await (0, query_1.parseMongodbQueryParams)(model, params);
    }
    async start() {
        if (this.#started)
            return;
        this.#started = true;
        mongoose_1.default.set('strictQuery', true);
        await mongoose_1.default.connect(instance_1.Instance.get().settings.mongoDbURI);
        await Promise.all([mongoose_1.default.connection, ...this.#connections].map((conn) => {
            return Object.values(conn.models)
                .map(async (model) => {
                await conn.db?.createCollection(model.collection.name, {
                    changeStreamPreAndPostImages: { enabled: true }
                });
            });
        }).flat());
    }
    async close() {
        await Promise.all(this.#connections.map(async (conn) => conn.close()));
        await mongoose_1.default.disconnect();
    }
}
exports.MongoDb = MongoDb;
