"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KafkaEventBus = void 0;
const kafkajs_1 = require("kafkajs");
const _1 = require(".");
const exit_1 = require("../exit");
const instance_1 = require("../instance");
const json_1 = require("../utils/json");
const utils_1 = require("../utils/utils");
class KafkaEventBus extends _1.EventBus {
    #client = new kafkajs_1.Kafka({
        clientId: instance_1.Instance.get().settings.eventColumnName,
        brokers: instance_1.Instance.get().settings.kafkaURIs,
        logLevel: kafkajs_1.logLevel.NOTHING
    });
    constructor() {
        super();
    }
    createPublisher(topic) {
        const publish = async (data) => {
            try {
                const producer = this.#client.producer();
                await producer.connect();
                await producer.send({
                    topic,
                    messages: [{ value: JSON.stringify(data) }],
                });
                return true;
            }
            catch {
                return false;
            }
        };
        return { publish };
    }
    createSubscriber(topic, onMessage, options = {}) {
        options = { ..._1.DefaultSubscribeOptions, ...options };
        let started = false;
        const subscribe = async () => {
            if (started)
                return;
            started = true;
            await this.#createTopic(topic);
            const groupId = options.fanout
                ? `${instance_1.Instance.get().settings.appId}-fanout-${utils_1.Random.string(10)}`
                : `${instance_1.Instance.get().settings.appId}-${topic}`;
            const consumer = this.#client.consumer({ groupId });
            await consumer.connect();
            await consumer.subscribe({ topic });
            await consumer.run({
                eachMessage: async ({ message }) => {
                    (0, exit_1.addWaitBeforeExit)((async () => {
                        if (!message.value)
                            return;
                        await onMessage((0, json_1.parseJSONValue)(message.value.toString())).catch();
                    })());
                },
            });
            if (options.fanout)
                (0, exit_1.addWaitBeforeExit)(async () => {
                    await consumer.disconnect();
                    await this.#deleteGroup(groupId);
                });
        };
        this._subscribers.push(subscribe);
        return { subscribe };
    }
    async #createTopic(topic) {
        const admin = this.#client.admin();
        await admin.createTopics({
            topics: [{ topic, numPartitions: 5 }],
        }).catch();
    }
    async #deleteGroup(groupId) {
        const admin = this.#client.admin();
        await admin.deleteGroups([groupId]).catch();
    }
}
exports.KafkaEventBus = KafkaEventBus;
