"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitEventBus = void 0;
const amqp_connection_manager_1 = __importDefault(require("amqp-connection-manager"));
const _1 = require(".");
const exit_1 = require("../exit");
const instance_1 = require("../instance");
const json_1 = require("../utils/json");
const utils_1 = require("../utils/utils");
class RabbitEventBus extends _1.EventBus {
    #client;
    #columnName = instance_1.Instance.get().settings.eventColumnName;
    constructor() {
        super();
        this.#client = amqp_connection_manager_1.default.connect([instance_1.Instance.get().settings.rabbitURI])
            .createChannel({
            json: false,
            setup: async (channel) => {
                await channel.assertExchange(this.#columnName, 'direct', { durable: true });
                await channel.prefetch(1);
            },
        });
    }
    createPublisher(topic) {
        const publish = async (data) => {
            return await this.#client.publish(this.#columnName, topic, JSON.stringify(data), { persistent: true });
        };
        return { publish };
    }
    createSubscriber(topic, onMessage, options = {}) {
        options = { ..._1.DefaultSubscribeOptions, ...options };
        let started = false;
        const subscribe = async () => {
            if (started)
                return;
            started = true;
            await this.#client.addSetup(async (channel) => {
                const queueName = options.fanout
                    ? `${instance_1.Instance.get().settings.appId}-fanout-${utils_1.Random.string(10)}`
                    : `${instance_1.Instance.get().settings.appId}-${topic}`;
                const { queue } = await channel.assertQueue(queueName, { durable: !options.fanout, exclusive: options.fanout });
                await channel.bindQueue(queue, this.#columnName, topic);
                channel.consume(queue, async (msg) => {
                    (0, exit_1.addWaitBeforeExit)((async () => {
                        if (!msg)
                            return;
                        try {
                            await onMessage((0, json_1.parseJSONValue)(msg.content.toString()));
                            channel.ack(msg);
                        }
                        catch {
                            channel.nack(msg);
                        }
                    })());
                }, { noAck: false });
            });
        };
        this._subscribers.push(subscribe);
        return { subscribe };
    }
}
exports.RabbitEventBus = RabbitEventBus;
