"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateJSONSchema = generateJSONSchema;
const ts_oas_1 = __importStar(require("ts-oas"));
const instance_1 = require("../instance");
const server_1 = require("../server");
const statusCodes = Object.entries(server_1.StatusCodes);
const fileSchema = { type: 'string', format: 'binary', example: 'binary file' };
function isFile(schema) {
    return schema.type === 'string' && schema.enum?.at(0) === 'equipped-file-schema';
}
function generateJSONSchema(patterns, paths, options) {
    const tsProgram = (0, ts_oas_1.createProgram)(paths, options?.tsConfig, options?.basePath);
    const logger = instance_1.Instance.createLogger();
    const tsoas = new ts_oas_1.default(tsProgram, {
        ref: false,
        nullableKeyword: false,
        schemaProcessor: (schema) => {
            if (isFile(schema))
                return fileSchema;
            if (Array.isArray(schema.items))
                schema.items = schema.items.map((s) => isFile(s) ? fileSchema : s);
            if (schema.anyOf)
                schema.anyOf = schema.anyOf.map((s) => isFile(s) ? fileSchema : s);
            if (schema.oneOf)
                schema.oneOf = schema.oneOf.map((s) => isFile(s) ? fileSchema : s);
            if (schema.allOf)
                schema.allOf = schema.allOf.map((s) => isFile(s) ? fileSchema : s);
            return schema;
        },
        ...(options?.options ?? {})
    });
    const jsonSchema = tsoas.getSchemas(patterns);
    return Object.entries(jsonSchema)
        .map(([name, { properties: def }]) => {
        try {
            const key = def?.key?.enum?.at(0) ?? name;
            const isApiDef = def?.__apiDef?.type === 'boolean' && def?.__apiDef?.enum?.[0] === true;
            const method = def?.method?.enum?.at?.(0)?.toLowerCase?.();
            if (!def || !isApiDef || !key || !method)
                return [undefined, undefined];
            const response = def.responses.properties ?? def.responses.anyOf?.reduce((acc, cur) => {
                if (cur.properties)
                    return { ...acc, ...cur.properties };
                return acc;
            }, {}) ?? undefined;
            if (response) {
                for (const [key, value] of Object.entries(response)) {
                    const status = statusCodes.find(([_, code]) => code.toString() === key);
                    if (status)
                        value['description'] = `${status[0]} Response`;
                }
            }
            const supportsBody = ['post', 'put', 'patch'].includes(method);
            const schema = {
                body: supportsBody ? def.body : undefined,
                params: def.params,
                querystring: def.query,
                headers: def.requestHeaders,
                response,
                operationId: key,
                summary: name,
            };
            return [key, schema];
        }
        catch (err) {
            logger.warn(`Error parsing ${name}: ${err.message}. Skipping route`);
            return [undefined, undefined];
        }
    })
        .reduce((acc, [path, schema]) => {
        if (!path || !schema)
            return acc;
        if (acc[path])
            logger.warn(`Duplicate route key '${path}' found for '${acc[path].summary}' & '${schema.summary}'. Make sure to use unique keys for all routes because only the last one will be used.`);
        acc[path] = stripEmptyObjects(schema);
        return acc;
    }, {});
}
function stripEmptyObjects(obj) {
    return Object.entries(obj).reduce((acc, [key, value]) => {
        if (!value || typeof value === 'object' && Object.keys(value).length === 0)
            return acc;
        return { ...acc, [key]: value };
    }, {});
}
