import http from 'http';
import { OpenAPIV3_1 } from 'openapi-types';
import supertest from 'supertest';
import { FastifySchema } from 'fastify';
import { Listener } from '../../listeners';
import { Defined } from '../../types';
import { Request } from '../requests';
import { Router } from '../routes';
import { Route } from '../types';
export type FullRoute = Required<Omit<Route, 'schema' | 'groups' | 'security' | 'descriptions' | 'hideSchema' | 'onError' | 'onSetupHandler' | '__def'>> & {
    schema: FastifySchema;
    onError?: Route['onError'];
};
type Schemas = Record<string, Defined<Route['schema']>>;
declare module 'openapi-types' {
    namespace OpenAPIV3 {
        interface Document {
            'x-tagGroups': {
                name: string;
                tags: string[];
            }[];
        }
        interface TagObject {
            'x-displayName': string;
        }
    }
}
export declare abstract class Server<Req = any, Res = any> {
    #private;
    protected server: http.Server;
    protected staticPath: string;
    protected settings: import("../../instance/settings").Settings;
    protected openapiJsonUrl: string;
    protected baseOpenapiDoc: OpenAPIV3_1.Document;
    protected abstract onLoad(): Promise<void>;
    protected abstract startServer(port: number): Promise<boolean>;
    protected abstract parse(req: Req, res: Res): Promise<Request>;
    protected abstract registerRoute(route: FullRoute): void;
    constructor(server: http.Server);
    get listener(): Listener;
    addRouter(...routers: Router[]): void;
    addRoute(...routes: Route[]): void;
    addSchema(...schemas: Schemas[]): void;
    load(): Promise<void>;
    test(): import("supertest/lib/agent")<supertest.SuperTestStatic.Test>;
    start(port: number): Promise<boolean>;
}
export {};
