import { Readable } from 'stream';
import { CustomError } from '../errors';
import { StorageFile } from '../storage';
import { DistributiveOmit, IsInTypeList } from '../types';
import { AuthUser, RefreshUser } from '../utils/authUser';
import { Api, FileSchema, GetApiPart, HeadersType, SupportedStatusCodes } from './types';
type HeaderKeys = 'AccessToken' | 'RefreshToken' | 'Referer' | 'ContentType' | 'UserAgent';
type IsFileOrFileArray<T> = T extends FileSchema ? StorageFile[] : T extends FileSchema[] ? StorageFile[] : T;
type ApiToBody<Def extends Api> = MappedUnion<Def['body']>;
type UnionMapper<T> = {
    [K in T extends infer P ? keyof P : never]: T extends infer P ? K extends keyof P ? {
        [Q in keyof T]: IsFileOrFileArray<T[Q]>;
    } : never : never;
};
type MappedUnion<T> = UnionMapper<T>[keyof UnionMapper<T>];
export declare class Request<Def extends Api = Api> {
    #private;
    readonly ip: string | undefined;
    readonly method: Def['method'];
    readonly path: string;
    readonly body: ApiToBody<Def>;
    readonly params: GetApiPart<Def, 'params'>;
    readonly query: GetApiPart<Def, 'query'>;
    readonly cookies: Record<string, any>;
    readonly rawBody: unknown;
    readonly headers: Record<HeaderKeys, string | undefined> & GetApiPart<Def, 'requestHeaders', true, {}> & HeadersType;
    authUser: null | AuthUser;
    refreshUser: null | RefreshUser;
    pendingError: null | CustomError;
    constructor({ ip, body, cookies, params, query, method, path, headers, files }: {
        ip: string | undefined;
        body: unknown;
        params: Def['params'];
        query: Def['query'];
        cookies: Record<string, any>;
        headers: Record<HeaderKeys, string | undefined> & GetApiPart<Def, 'requestHeaders', true, {}> & HeadersType;
        files: Record<string, StorageFile[]>;
        method: Def['method'];
        path: string;
    });
    pipe(stream: Readable, opts?: {
        headers?: GetApiPart<Def, 'responseHeaders'>;
        status?: GetApiPart<Def, 'defaultStatusCode'>;
    }): Response<Readable, SupportedStatusCodes, HeadersType>;
    res(params: DistributiveOmit<RequestParams<Def['response'], GetApiPart<Def, 'defaultStatusCode'>, GetApiPart<Def, 'responseHeaders'>>, 'piped'>): Response<Def["response"], import("../types").IsType<Exclude<Def["defaultStatusCode"], undefined>, unknown> extends true ? SupportedStatusCodes : Exclude<Def["defaultStatusCode"], undefined>, import("../types").IsType<Exclude<Def["responseHeaders"], undefined>, unknown> extends true ? HeadersType : Exclude<Def["responseHeaders"], undefined>>;
    error(params: DistributiveOmit<RequestParams<CustomError['serializedErrors'], CustomError['statusCode'], HeadersType>, 'piped'>): Response<{
        message: string;
        field?: string;
    }[], import("./types").BadStatusCodes, HeadersType>;
}
type RequestParams<T, S extends SupportedStatusCodes, H extends HeadersType> = {
    body: T;
    piped?: boolean;
} & (IsInTypeList<S, [SupportedStatusCodes, 200]> extends true ? {
    status?: S;
} : {
    status: S;
}) & (IsInTypeList<H, [HeadersType]> extends true ? {
    headers?: H;
} : {
    headers: H;
});
export declare class Response<T, S extends SupportedStatusCodes, H extends HeadersType> {
    readonly body: T | undefined;
    readonly status: S;
    readonly headers: H;
    readonly piped: boolean;
    constructor({ body, status, headers, piped }: RequestParams<T, S, H>);
    get shouldJSONify(): boolean;
}
export {};
