"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Response = exports.Request = void 0;
const json_1 = require("../utils/json");
class Request {
    ip;
    method;
    path;
    body;
    params;
    query;
    cookies;
    rawBody;
    headers;
    authUser = null;
    refreshUser = null;
    pendingError = null;
    constructor({ ip, body, cookies, params, query, method, path, headers, files }) {
        this.ip = ip;
        this.method = method;
        this.path = path;
        this.rawBody = body;
        this.body = Object.assign(Object.fromEntries(Object.entries(body && typeof body === 'object' ? body : { raw: body })
            .map(([key, value]) => [key, (0, json_1.parseJSONValue)(value)])), files);
        this.cookies = cookies;
        this.params = params;
        this.query = Object.fromEntries(Object.entries(query && typeof body === 'object' ? query : {})
            .map(([key, val]) => [key, this.#parseQueryStrings(val)]));
        if (this.query?.['auth'])
            delete this.query['auth'];
        if (this.query?.['authType'])
            delete this.query['authType'];
        this.headers = headers;
    }
    #parseQueryStrings(value) {
        if (Array.isArray(value))
            return value.map(this.#parseQueryStrings);
        if (typeof value === 'string')
            return (0, json_1.parseJSONValue)(value);
        return value;
    }
    pipe(stream, opts = {}) {
        return new Response({ ...opts, piped: true, body: stream });
    }
    res(params) {
        return new Response({ ...params, piped: false });
    }
    error(params) {
        return new Response({ ...params, piped: false });
    }
}
exports.Request = Request;
class Response {
    body;
    status;
    headers;
    piped;
    constructor({ body, status = 200, headers = {}, piped = false }) {
        this.body = body;
        this.status = status;
        this.headers = headers;
        this.piped = piped;
        if (!this.piped) {
            const contentType = Object.keys(this.headers).find((key) => key.toLowerCase() === 'content-type');
            if (!contentType)
                this.headers['Content-Type'] = 'application/json';
        }
    }
    get shouldJSONify() {
        return this.body === null || this.body === undefined;
    }
}
exports.Response = Response;
