"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Router = exports.groupRoutes = exports.cleanPath = void 0;
const valleyed_1 = require("valleyed");
const types_1 = require("./types");
const cleanPath = (path) => {
    let cleaned = path.replace(/(\/\s*)+/g, '/');
    if (!cleaned.startsWith('/'))
        cleaned = `/${cleaned}`;
    if (cleaned !== '/' && cleaned.endsWith('/'))
        cleaned = cleaned.slice(0, -1);
    return cleaned;
};
exports.cleanPath = cleanPath;
const groupRoutes = (config, routes) => routes
    .map((route) => ({
    ...config,
    ...route,
    path: (0, exports.cleanPath)(`${config.path}/${route.path}`),
    groups: [...(config.groups ?? []), ...(route.groups ?? [])],
    middlewares: [...(config.middlewares ?? []), ...(route.middlewares ?? [])],
    security: [...(config.security ?? []), ...(route.security ?? [])],
}));
exports.groupRoutes = groupRoutes;
class Router extends valleyed_1.ClassPropertiesWrapper {
    #config = { path: '' };
    #routes = [];
    #children = [];
    constructor(config) {
        const methodImpls = Object.fromEntries(Object.values(types_1.Methods).map((method) => [method, (route) => this.#addRoute(method, route)]));
        super(methodImpls);
        if (config)
            this.#config = config;
    }
    #addRoute(method, routeConfig, collection = this.#routes) {
        return (handler) => {
            const route = (0, exports.groupRoutes)(this.#config, [{ ...routeConfig, method, handler }])[0];
            collection.push(route);
            return route;
        };
    }
    add(...routes) {
        const mapped = (0, exports.groupRoutes)(this.#config, routes);
        this.#routes.push(...mapped);
    }
    nest(...routers) {
        routers.forEach((router) => this.#children.push(router));
    }
    get routes() {
        const routes = [...this.#routes];
        this.#children.forEach((child) => {
            child.routes.forEach((route) => {
                this.#addRoute(route.method, route, routes)(route.handler);
            });
        });
        return routes;
    }
}
exports.Router = Router;
