"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.signinWithFacebook = exports.signinWithApple = exports.signinWithGoogle = void 0;
const axios_1 = __importDefault(require("axios"));
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const jwks_rsa_1 = __importDefault(require("jwks-rsa"));
const signinWithGoogle = async (idToken) => {
    const authUrl = `https://oauth2.googleapis.com/tokeninfo?id_token=${idToken}`;
    const { data } = await axios_1.default.get(authUrl).catch((err) => {
        const message = err?.response?.data?.error;
        throw new Error(message ? 'Invalid id token' : 'Something unexpected happened');
    });
    data.first_name = data.given_name;
    data.last_name = data.family_name;
    return data;
};
exports.signinWithGoogle = signinWithGoogle;
const signinWithApple = async (idToken) => {
    try {
        const APPLE_BASE = 'https://appleid.apple.com';
        const json = jsonwebtoken_1.default.decode(idToken, { complete: true });
        if (!json?.header)
            throw new Error('');
        const { kid, alg } = json.header;
        const publicKey = await (0, jwks_rsa_1.default)({ jwksUri: `${APPLE_BASE}/auth/keys`, cache: true })
            .getSigningKey(kid).then((key) => key.getPublicKey()).catch(() => null);
        if (!publicKey)
            throw new Error('');
        const data = jsonwebtoken_1.default.verify(idToken, publicKey, { algorithms: [alg] });
        if (!data)
            throw new Error('');
        if (data.iss !== APPLE_BASE)
            throw new Error('');
        if (data.exp * 1000 < Date.now())
            throw new Error('expired idToken');
        return data;
    }
    catch (err) {
        throw new Error(err.message || 'Invalid idToken');
    }
};
exports.signinWithApple = signinWithApple;
const signinWithFacebook = async (accessToken, fields = []) => {
    fields = [...new Set([...fields, 'name', 'picture', 'email'])];
    const { data } = await axios_1.default.request({
        method: 'get',
        url: 'https://graph.facebook.com/v15.0/me',
        params: {
            fields: fields.join(','),
            access_token: accessToken
        }
    }).catch((err) => {
        const message = err?.response?.data?.error?.message;
        throw new Error(message ? 'Invalid access token' : 'Something unexpected happened');
    });
    const isValidData = fields.every((key) => key in data);
    if (!isValidData)
        throw new Error('Incomplete scope for access token');
    data.email_verified = 'true';
    return data;
};
exports.signinWithFacebook = signinWithFacebook;
