"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exchangeOldForNewTokens = exports.deleteCachedRefreshToken = exports.deleteCachedAccessToken = exports.getCachedRefreshToken = exports.getCachedAccessToken = exports.verifyRefreshToken = exports.verifyAccessToken = exports.makeRefreshToken = exports.makeAccessToken = void 0;
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const errors_1 = require("../errors");
const instance_1 = require("../instance");
const server_1 = require("../server");
const getAccessTokenKey = (userId) => `${userId}-access-token`;
const getRefreshTokenKey = (userId) => `${userId}-refresh-token`;
const makeAccessToken = async (payload) => {
    const token = jsonwebtoken_1.default.sign(payload, instance_1.Instance.get().settings.accessTokenKey, { expiresIn: instance_1.Instance.get().settings.accessTokenTTL });
    await instance_1.Instance.get().cache.set(getAccessTokenKey(payload.id), token, instance_1.Instance.get().settings.accessTokenTTL);
    return token;
};
exports.makeAccessToken = makeAccessToken;
const makeRefreshToken = async (payload) => {
    const token = jsonwebtoken_1.default.sign(payload, instance_1.Instance.get().settings.refreshTokenKey, { expiresIn: instance_1.Instance.get().settings.refreshTokenTTL });
    await instance_1.Instance.get().cache.set(getRefreshTokenKey(payload.id), token, instance_1.Instance.get().settings.refreshTokenTTL);
    return token;
};
exports.makeRefreshToken = makeRefreshToken;
const verifyAccessToken = async (token) => {
    try {
        const user = jsonwebtoken_1.default.verify(token, instance_1.Instance.get().settings.accessTokenKey);
        if (!user)
            throw new errors_1.NotAuthenticatedError();
        const cachedToken = await (0, exports.getCachedAccessToken)(user.id);
        if (token && token !== cachedToken)
            throw new errors_1.AccessTokenExpired();
        return user;
    }
    catch (err) {
        if (err instanceof errors_1.AccessTokenExpired)
            throw err;
        if (err instanceof jsonwebtoken_1.default.TokenExpiredError)
            throw new errors_1.AccessTokenExpired();
        else
            throw new errors_1.NotAuthenticatedError();
    }
};
exports.verifyAccessToken = verifyAccessToken;
const verifyRefreshToken = async (token) => {
    try {
        const user = jsonwebtoken_1.default.verify(token, instance_1.Instance.get().settings.refreshTokenKey);
        if (!user)
            throw new errors_1.NotAuthenticatedError();
        return user;
    }
    catch {
        throw new errors_1.NotAuthenticatedError();
    }
};
exports.verifyRefreshToken = verifyRefreshToken;
const getCachedAccessToken = async (userId) => instance_1.Instance.get().cache.get(getAccessTokenKey(userId));
exports.getCachedAccessToken = getCachedAccessToken;
const getCachedRefreshToken = async (userId) => instance_1.Instance.get().cache.get(getRefreshTokenKey(userId));
exports.getCachedRefreshToken = getCachedRefreshToken;
const deleteCachedAccessToken = async (userId) => instance_1.Instance.get().cache.delete(getAccessTokenKey(userId));
exports.deleteCachedAccessToken = deleteCachedAccessToken;
const deleteCachedRefreshToken = async (userId) => instance_1.Instance.get().cache.delete(getRefreshTokenKey(userId));
exports.deleteCachedRefreshToken = deleteCachedRefreshToken;
const exchangeOldForNewTokens = async ({ accessToken, refreshToken }, makeTokens) => {
    const authUser = await (0, exports.verifyAccessToken)(accessToken).catch((err) => {
        const error = err;
        if (error.statusCode === server_1.StatusCodes.AccessTokenExpired)
            return null;
        else
            throw err;
    });
    if (authUser)
        return await makeTokens(authUser.id);
    const refreshUser = await (0, exports.verifyRefreshToken)(refreshToken);
    return await makeTokens(refreshUser.id);
};
exports.exchangeOldForNewTokens = exchangeOldForNewTokens;
