import * as Validate from 'valleyed';
import { ExtractI } from 'valleyed/lib/api/base';
import { VCore } from 'valleyed/lib/api/core';
import { VObject } from 'valleyed/lib/api/objects';
import { StorageFile } from '../storage';
type Phone = {
    code: string;
    number: string;
};
export declare const Schema: {
    is: <T>(value: T, comparer?: (val: any, comp: T) => boolean, err?: string) => VCore<T>;
    in: <T>(values: Readonly<T[]>, comparer?: (val: any, arrayItem: T) => boolean, err?: string) => VCore<T>;
    or: <T extends VCore<any>[]>(options: T, err?: string | undefined) => import("valleyed/lib/api/junctions").VOr<T>;
    and: <I>(options: VCore<I>[], err?: string | undefined) => import("valleyed/lib/api/junctions").VAnd<I>;
    string: (err?: string | undefined) => import("valleyed/lib/api/strings").VString;
    discriminate: <D extends Record<string, VCore<any>>>(discriminator: (val: import("valleyed/lib/api/base").ExtractI<D[keyof D]>) => string, schemas: D, err?: string | undefined) => import("valleyed/lib/api/junctions").VDiscriminator<D>;
    number: (err?: string | undefined) => import("valleyed/lib/api/numbers").VNumber;
    boolean: (err?: string | undefined) => import("valleyed/lib/api/booleans").VBoolean;
    time: <T extends import("valleyed/lib/api/times").Timeable = import("valleyed/lib/api/times").Timeable>(err?: string | undefined) => import("valleyed/lib/api/times").VTime<T>;
    file: (err?: string | undefined) => import("valleyed/lib/api/files").VFile;
    array: <I>(comparer: VCore<I>, err?: string | undefined) => import("valleyed/lib/api/arrays").VArray<I>;
    tuple: <T extends ReadonlyArray<VCore<any>>>(schema: readonly [...T], err?: string | undefined) => import("valleyed/lib/api/tuples").VTuple<T>;
    object: <T extends Record<string, VCore<any>>>(schema: T, trim?: boolean | undefined, err?: string | undefined) => VObject<T>;
    record: <VI>(vCom: VCore<VI>, err?: string | undefined) => import("valleyed/lib/api/records").VRecord<VI>;
    map: <KI, VI>(kCom: VCore<KI>, vCom: VCore<VI>, err?: string | undefined) => import("valleyed/lib/api/records").VMap<KI, VI>;
    null: (err?: string) => VCore<null>;
    undefined: (err?: string) => VCore<undefined>;
    instanceof: <T>(classDef: new () => T, err?: string) => VCore<T>;
    any: <T = any>() => VCore<T>;
    force: {
        string: (err?: string | undefined) => import("valleyed/lib/api/strings").VString;
        number: (err?: string | undefined) => import("valleyed/lib/api/numbers").VNumber;
        boolean: (err?: string | undefined) => import("valleyed/lib/api/booleans").VBoolean;
        time: (err?: string | undefined) => import("valleyed/lib/api/times").VTime<Date>;
    };
};
export declare const Validation: {
    isNotTruncated: (error?: string) => Validate.Rule<StorageFile>;
    isValidPhone: (error?: string) => Validate.Rule<Phone>;
    Validator: typeof Validate.Validator;
    VCore: typeof Validate.VCore;
    v: {
        is: <T>(value: T, comparer?: (val: any, comp: T) => boolean, err?: string) => VCore<T>;
        in: <T>(values: Readonly<T[]>, comparer?: (val: any, arrayItem: T) => boolean, err?: string) => VCore<T>;
        or: <T extends VCore<any>[]>(options: T, err?: string | undefined) => import("valleyed/lib/api/junctions").VOr<T>;
        and: <I>(options: VCore<I>[], err?: string | undefined) => import("valleyed/lib/api/junctions").VAnd<I>;
        string: (err?: string | undefined) => import("valleyed/lib/api/strings").VString;
        discriminate: <D extends Record<string, VCore<any>>>(discriminator: (val: import("valleyed/lib/api/base").ExtractI<D[keyof D]>) => string, schemas: D, err?: string | undefined) => import("valleyed/lib/api/junctions").VDiscriminator<D>;
        number: (err?: string | undefined) => import("valleyed/lib/api/numbers").VNumber;
        boolean: (err?: string | undefined) => import("valleyed/lib/api/booleans").VBoolean;
        time: <T extends import("valleyed/lib/api/times").Timeable = import("valleyed/lib/api/times").Timeable>(err?: string | undefined) => import("valleyed/lib/api/times").VTime<T>;
        file: (err?: string | undefined) => import("valleyed/lib/api/files").VFile;
        array: <I>(comparer: VCore<I>, err?: string | undefined) => import("valleyed/lib/api/arrays").VArray<I>;
        tuple: <T extends ReadonlyArray<VCore<any>>>(schema: readonly [...T], err?: string | undefined) => import("valleyed/lib/api/tuples").VTuple<T>;
        object: <T extends Record<string, VCore<any>>>(schema: T, trim?: boolean | undefined, err?: string | undefined) => VObject<T>;
        record: <VI>(vCom: VCore<VI>, err?: string | undefined) => import("valleyed/lib/api/records").VRecord<VI>;
        map: <KI, VI>(kCom: VCore<KI>, vCom: VCore<VI>, err?: string | undefined) => import("valleyed/lib/api/records").VMap<KI, VI>;
        null: (err?: string) => VCore<null>;
        undefined: (err?: string) => VCore<undefined>;
        instanceof: <T>(classDef: new () => T, err?: string) => VCore<T>;
        any: <T = any>() => VCore<T>;
        force: {
            string: (err?: string | undefined) => import("valleyed/lib/api/strings").VString;
            number: (err?: string | undefined) => import("valleyed/lib/api/numbers").VNumber;
            boolean: (err?: string | undefined) => import("valleyed/lib/api/booleans").VBoolean;
            time: (err?: string | undefined) => import("valleyed/lib/api/times").VTime<Date>;
        };
    };
    isEqualTo: <Type>(compare: Type, comparer?: (val: any, compare: Type) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Type>;
    isNotEqualTo: <Compare, Type>(compare: Compare, comparer?: (val: Type, compare: Compare) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Type>;
    arrayContains: <Type>(array: Readonly<Type[]>, comparer?: (val: any, arrayItem: Type) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Type>;
    arrayNotContains: <Compare, Type>(array: Readonly<Compare[]>, comparer?: (val: Type, arrayItem: Compare) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Type>;
    isString: (error?: string) => import("valleyed/lib/utils/rules").Rule<any>;
    isLengthOf: (length: number, stripHTMLTags?: boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<string>;
    isMinOf: (length: number, stripHTMLTags?: boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<string>;
    isMaxOf: (length: number, stripHTMLTags?: boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<string>;
    isEmail: (error?: string) => import("valleyed/lib/utils/rules").Rule<string>;
    isUrl: (error?: string) => import("valleyed/lib/utils/rules").Rule<string>;
    isNumber: (error?: string) => import("valleyed/lib/utils/rules").Rule<any>;
    isInt: (error?: string) => import("valleyed/lib/utils/rules").Rule<any>;
    isMoreThan: (compare: number, error?: string) => import("valleyed/lib/utils/rules").Rule<number>;
    isMoreThanOrEqualTo: (compare: number, error?: string) => import("valleyed/lib/utils/rules").Rule<number>;
    isLessThan: (compare: number, error?: string) => import("valleyed/lib/utils/rules").Rule<number>;
    isLessThanOrEqualTo: (compare: number, error?: string) => import("valleyed/lib/utils/rules").Rule<number>;
    isArray: (error?: string) => import("valleyed/lib/utils/rules").Rule<any>;
    hasLengthOf: <Type>(length: number, error?: string) => import("valleyed/lib/utils/rules").Rule<Type[]>;
    hasMinOf: <Type>(length: number, error?: string) => import("valleyed/lib/utils/rules").Rule<Type[]>;
    hasMaxOf: <Type>(length: number, error?: string) => import("valleyed/lib/utils/rules").Rule<Type[]>;
    isArrayOf: <Type>(comparer: (cur: Type, idx: number) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Type[]>;
    isUndefined: (error?: string) => import("valleyed/lib/utils/rules").Rule<undefined>;
    isNull: (error?: string) => import("valleyed/lib/utils/rules").Rule<null>;
    isBoolean: (error?: string) => import("valleyed/lib/utils/rules").Rule<boolean>;
    isInstanceOf: <T>(classDef: new () => T, error?: string) => import("valleyed/lib/utils/rules").Rule<T>;
    fileMimeTypes: string[];
    videoMimeTypes: string[];
    audioMimeTypes: string[];
    imageMimeTypes: string[];
    isImage: (error?: string) => import("valleyed/lib/utils/rules").Rule<import("valleyed/lib/types").File>;
    isAudio: (error?: string) => import("valleyed/lib/utils/rules").Rule<import("valleyed/lib/types").File>;
    isVideo: (error?: string) => import("valleyed/lib/utils/rules").Rule<import("valleyed/lib/types").File>;
    isFile: (error?: string) => import("valleyed/lib/utils/rules").Rule<import("valleyed/lib/types").File>;
    isTuple: <T extends ReadonlyArray<(v: any, idx: number) => boolean>>(comparer: Validate.GetMap<T>, error?: string) => import("valleyed/lib/utils/rules").Rule<{ [K in keyof T]: T[K] extends (v: infer V, idx: number) => boolean ? V : never; }>;
    isRecord: <V>(com: (cur: V) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Record<string, V>>;
    isMap: <K, V>(kCom: (cur: K) => boolean, vCom: (cur: V) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<Map<K, V>>;
    isCustom: <T>(validity: (v: T) => boolean, error?: string) => import("valleyed/lib/utils/rules").Rule<T>;
    isTime: <T extends Validate.Timeable>(error?: string) => import("valleyed/lib/utils/rules").Rule<T>;
    isLaterThan: <T extends Validate.Timeable>(compare: Validate.Timeable, error?: string) => import("valleyed/lib/utils/rules").Rule<T>;
    isEarlierThan: <T extends Validate.Timeable>(compare: Validate.Timeable, error?: string) => import("valleyed/lib/utils/rules").Rule<T>;
    ClassPropertiesWrapper: typeof Validate.ClassPropertiesWrapper;
    Differ: typeof Validate.Differ;
    capitalize: (text: string) => string;
    stripHTML: (html: string) => string;
    trimToLength: (body: string, length: number) => string;
    extractUrls: (text: string) => {
        original: string;
        normalized: string;
    }[];
    formatNumber: (num: number, dp?: number) => string;
    pluralize: (count: number, singular: string, plural: string) => string;
    getRandomValue: () => string;
    groupBy: <Type, Unique extends string | number>(array: Array<Type>, func: (item: Type) => Unique) => {
        key: Unique;
        values: Type[];
    }[];
    getAlphabet: (num: number) => string;
    addToArray: <T>(array: T[], item: T, getKey: (a: T) => any, getComparer: (a: T) => number | string, asc?: boolean) => T[];
    divideByZero: (num: number, den: number) => number;
    getPercentage: (num: number, den: number) => number;
    getRandomSample: <Type>(population: Array<Type>, n: number) => Type[];
    shuffleArray: <Type>(array: Array<Type>) => Type[];
    chunkArray: <T>(arr: T[], size: number) => T[][];
    wrapInTryCatch: <T>(fn: () => T, defaultValue?: T) => T | undefined;
    compareTwoStrings: (first: string, second: string) => number;
    Geohash: typeof Validate.Geohash;
    isValid: <T>(value: T, ignored?: boolean) => {
        valid: true;
        errors: string[];
        value: T;
        ignored: boolean;
    };
    isInvalid: <T>(errors: string[], value: T) => {
        valid: false;
        errors: string[];
        value: unknown;
        ignored: boolean;
    };
    makeRule: <T>(func: Validate.Rule<T>) => Validate.Rule<T>;
    makeSanitizer: <T>(func: Validate.Sanitizer<T>) => (val: T) => T;
    check: <T>(value: T, rules: Validate.Rule<T>[], options?: Partial<Validate.Options>) => ReturnType<Validate.Rule<T>>;
};
export declare function validate<T extends Record<string, VCore<any>>>(schema: T, value: unknown): ExtractI<VObject<T>>;
export declare function validate<T extends VCore<any>>(schema: T, value: unknown): ExtractI<T>;
export declare const Hash: {
    hash: (password: string) => Promise<string>;
    compare: (plainPassword: string, hashed: string) => Promise<boolean>;
};
declare module 'valleyed/lib/types' {
    interface File extends StorageFile {
    }
}
export {};
