"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hash = exports.Validation = exports.Schema = void 0;
exports.validate = validate;
const bcrypt = __importStar(require("bcryptjs"));
const Validate = __importStar(require("valleyed"));
const errors_1 = require("../errors");
const instance_1 = require("../instance");
const isNotTruncated = (error) => Validate.makeRule((file) => {
    const val = file;
    error = error ?? `is larger than allowed limit of ${instance_1.Instance.get().settings.maxFileUploadSizeInMb}mb`;
    const valid = val ? !val.isTruncated : true;
    return valid ? Validate.isValid(val) : Validate.isInvalid([error], val);
});
const isValidPhone = (error) => Validate.makeRule((value) => {
    return Validate.v.object({
        code: Validate.v.string().custom((val) => {
            return val.startsWith('+') &&
                Validate.v.force.number(val.slice(1)).parse(val).valid;
        }, error ?? 'invalid phone code'),
        number: Validate.v.force.number(error ?? 'invalid phone number').transform((val) => val.toString())
    }).parse(value);
});
const file = Validate.v.file;
Validate.v.file = (...args) => file(...args).addRule(isNotTruncated());
exports.Schema = Validate.v;
exports.Validation = { ...Validate, isNotTruncated, isValidPhone };
function validate(schema, value) {
    const validator = schema instanceof Validate.VCore ? schema : Validate.v.object(schema);
    const validity = validator.parse(value);
    if (validity.valid)
        return validity.value;
    const errorsObject = validity.errors
        .map((error) => {
        const splitKey = ': ';
        const [field, ...rest] = error.split(splitKey);
        return { field, message: rest.join(splitKey) };
    }).reduce(((acc, cur) => {
        if (acc[cur.field])
            acc[cur.field].push(cur.message);
        else
            acc[cur.field] = [cur.message];
        return acc;
    }), {});
    throw new errors_1.ValidationError(Object.entries(errorsObject)
        .map(([key, value]) => ({ field: key, messages: value })));
}
const hash = async (password) => {
    password = password.trim();
    if (!password)
        return '';
    return await bcrypt.hash(password, instance_1.Instance.get().settings.hashSaltRounds);
};
const compare = async (plainPassword, hashed) => {
    plainPassword = plainPassword.trim();
    if (!plainPassword && plainPassword === hashed)
        return true;
    return await bcrypt.compare(plainPassword, hashed);
};
exports.Hash = { hash, compare };
